import os, time, pygame, photoboite

class Button:
    def __init__(self, rect):
        self.rect = rect
        self.pressed = False
        self.enabled = True

    def draw_on(self, screen):
        pass

    def is_pressed(self, pos):
        return self.rect.collidepoint(pos)

    def press(self):
        self.pressed = True

    def unpress(self):
        self.pressed = False
        self.run()

    def run(self):
        pass

class ConfigButton(Button):
    def __init__(self, rect, config):
        self.config = config

        self.count = 0
        self.timestamp = 0

        super(ConfigButton, self).__init__(rect)

    def run(self):
        if time.time() - self.timestamp > 0.5:
            self.count = 1
            self.timestamp = time.time()
        else:
            self.count += 1
            self.timestamp = time.time()

        if self.count >= 5:
            self.config.activate()


class InfoButton(Button):
    def __init__(self, rect, photoboite):
        self.count = 0
        self.timestamp = 0
        self.photoboite = photoboite

        super(InfoButton, self).__init__(rect)

        self.enabled = False
        self.cache = False

        self.font = pygame.font.Font(None, 25)
        self.logo = pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../../images/raspberrypi.png').convert_alpha()

    def date(self):
        if not self.cache:
            self.date_string = str(time.strftime('%Y-%m-%d %H:%M:%S'))
        return self.date_string

    def disk_usage(self):
        if not self.cache:
            st = os.statvfs('/')
            free = round(st.f_bavail * st.f_frsize / 1073741824)
            total = round(st.f_blocks * st.f_frsize / 1073741824)
            used = round((st.f_blocks - st.f_bfree) * st.f_frsize / 1073741824)

            self.disk_usage_string = str(used) + "G / " + str(total) + "G (free: " + str(free) + "G)"

        return self.disk_usage_string

    def draw_on(self, screen):
        if self.enabled:
            screen.blit(self.logo, self.logo.get_rect())

            top = self.logo.get_rect().height + 10

            texts = [
                "DATE: " + self.date(),
                "COUNT: " + str(self.photoboite.capture.capture_button.count),
                "FPS: " + str(self.photoboite.clock.get_fps()),
                "DISK USAGE:" + self.disk_usage()
            ]

            i = 0
            for text in texts:
                size = self.font.size(text)
                ren = self.font.render(text, True, (255, 255, 255))
                screen.blit(ren, (0, top + i * 20))

                i += 1

    def run(self):
        if time.time() - self.timestamp > 0.5:
            self.count = 1
        else:
            self.count += 1

        self.timestamp = time.time()

        if self.count >= 2:
            self.enabled = not self.enabled


class PowerButton(Button):
    def __init__(self, rect):
        self.count = 0
        self.timestamp = 0

        super(PowerButton, self).__init__(rect)

    def run(self):
        if time.time() - self.timestamp > 0.5:
            self.count = 1
            self.timestamp = time.time()
        else:
            self.count += 1
            self.timestamp = time.time()

        if self.count >= 5:
            os.system('sudo poweroff')

class CaptureButton(Button):
    countdown = 3

    def __init__(self, size, topleft, photoboite):
        self.up_image = pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../../images/button-up.png').convert_alpha()
        self.up_image = pygame.transform.smoothscale(self.up_image, (size, size))

        self.down_image = pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../../images/button-down.png').convert_alpha()
        self.down_image = pygame.transform.smoothscale(self.down_image, (size, size))

        rect = self.up_image.get_rect()
        rect.topleft = topleft

        super(CaptureButton, self).__init__(rect)
        self.photoboite = photoboite
        self.elapsed = 0
        self.capture_mode = False
        self.count = 0

        self.background_rect = pygame.Rect(0, self.photoboite.screen().get_height() - 80, self.photoboite.screen().get_width(), 80)
        self.background = pygame.Surface((self.background_rect.width, self.background_rect.height))
        self.background.fill((0, 0, 0))
        self.background.set_alpha(150)

        self.font = pygame.font.Font(None, 80)

    def draw_on(self, screen):
        if self.enabled:
            if self.pressed:
                screen.blit(self.down_image, self.rect)
            else:
                screen.blit(self.up_image, self.rect)
        if self.capture_mode and self.elapsed < CaptureButton.countdown:
            screen.blit(self.background, self.background_rect.topleft)

            text = str(CaptureButton.countdown - self.elapsed)
            size = self.font.size(text)
            ren = self.font.render(text, True, (255, 255, 255))
            ren.set_alpha(150)

            topleft = (self.rect.topleft[0] + self.rect.width / 2 - size[0] / 2, self.rect.topleft[1] + self.rect.height / 2 - size[1] / 2)
            top = self.background_rect.top + self.background_rect.height / 2 - size[1] / 2
            left = self.background_rect.left + self.background_rect.width / 2 - size[0] / 2
            screen.blit(ren, (left, top))

    def run(self):
        pygame.event.set_blocked((pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP))
        self.capture_mode = True
        self.enabled = False

        capture = photoboite.Capture(self.count)
        self.count += 1

        for i in range(3):
            wait = True
            start = time.time()
            while wait:
                self.elapsed = int(time.time() - start)
                if self.elapsed >= CaptureButton.countdown:
                    wait = False

                self.photoboite.event()
                self.photoboite.draw()
                pygame.display.flip()

            capture.take(self.photoboite.camera)
            pygame.time.wait(600)

        self.photoboite.screen().fill((200,200,200))

        width = self.photoboite.screen().get_width()
        height = self.photoboite.screen().get_height()

        for photo in capture.photos:
            photo_width = photo.capture.get_width()
            photo_height = photo.capture.get_height()

            lefttop = ((width - photo_width) / 2, (height - photo_height) / 2)
            self.photoboite.screen().blit(photo.capture, lefttop)
            pygame.display.flip()
            pygame.time.wait(800)
            self.photoboite.event()

        pygame.time.wait(200)

        pygame.event.set_allowed((pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP))
        self.capture_mode = False
        self.enabled = True
        self.photoboite.sleep.move()
